<?

include "htaccess.conf";

$status = "";
$users = array();
$groups = array();
$dirs = array();
$username_ereg = '[A-Za-z0-9_.@=+-]{1,99}';
$fieldname_codes = array( "@", ".", "=", "+", "-" );

function ok_username( $username ) {
  return ereg( "^$GLOBALS[username_ereg]$", $username );
}

function ok_groupname( $groupname ) {
  return ereg( "^[A-Za-z0-9_]{1,20}$", $groupname );
}

function ok_dirname( $dirname ) {
  return ereg( "^(/[A-Za-z0-9_]{1,40}){1,10}$", $dirname );
}

function ok_password( $password ) {
  return ereg( "^([A-Za-z0-9_ .,:;@=/*+-]{1,40})$", $password );
}

function encode_fieldname( $fieldname ) {
  foreach( $GLOBALS["fieldname_codes"] as $code => $char ) {
    $fieldname = str_replace( $char, "_".$code."_", $fieldname );
  }
  return $fieldname;
}

function get_users() {
  $users = $GLOBALS["users"];
  if( count( $users ) < 1 ) {
    $file = fopen( $GLOBALS["htpassfile"], "r" );
    while( $file and ! feof( $file ) ) {
      $row = fgets( $file );
      if( ereg( "^($GLOBALS[username_ereg]):", $row, $regs ) )
        $users[] = $regs[1];
    }
    fclose( $file );
    $GLOBALS["users"] = $users;
  }
  return $users;
}

# password encryption based on http://fi.php.net/manual/en/function.crypt.php

function get_salt_chars() {
  $salt_chars = Array();
  for( $c = ord("0"); $c <= ord("9"); ++$c) {
	$salt_chars[] = $c;
  }
  for( $c = ord("A"); $c <= ord("Z"); ++$c) {
	$salt_chars[] = $c;
  }
  for( $c = ord("a"); $c <= ord("z"); ++$c) {
	$salt_chars[] = $c;
  }
  return $salt_chars;
}

function make_salt_string( $len ) {
  $salt_chars = get_salt_chars();
  $salt = '';
  while( $len-- > 0 ) {
    $salt .= chr( $salt_chars[ rand( 0, count($salt_chars) - 1 ) ] );
  }
  return $salt;
}

function make_salt( $type ) {
  if($type == 'BLOWFISH') {
  	return '$2a$05$' . make_salt_string(8) . '$';
  } elseif($type == 'MD5') {
  	return '$1$' . make_salt_string(8) . '$';
  } elseif($type == 'EXT_DES') {
    return make_salt_string(9);
  }
  return make_salt_string(2);
}

function control_crypt( $password ) {
  $salt_type = $GLOBALS["crypt_salt_type"];
  if( $salt_type ) {
    $salt = make_salt( $salt_type );
    return crypt( $password, $salt );
  } else {
    return crypt( $password );
  }
}

function write_user( $username, $password ) {
  $from = fopen( $GLOBALS["htpassfile"], "r" );
  $to = fopen( "$GLOBALS[htpassfile]+", "w" );
  if( ! $to ) {
    return err( "Failed to write password file" );
  }
  if( $password ) {
    $new_row = "$username:" . control_crypt($password) . "\n";
  } else {
    $new_row = ""; # delete user
  }
  $written = false;
  while( $from and ! feof( $from ) ) {
      $row = fgets( $from );
      if( ereg( "^$username:", $row ) ) {
        # replace existing user
	$row = $new_row;
	$written = true;
      }
      fputs( $to, $row );
  }
  # append new user
  if( ! $written ) fputs( $to, $new_row );
  fclose( $to );
  fclose( $from );
  if( file_exists( "$GLOBALS[htpassfile]-" ) )
    unlink( "$GLOBALS[htpassfile]-" );
  rename( $GLOBALS["htpassfile"], "$GLOBALS[htpassfile]-" );
  rename( "$GLOBALS[htpassfile]+", $GLOBALS["htpassfile"] );
}

function get_groups() {
  $groups = $GLOBALS["groups"];
  if( count( $groups ) < 1 ) {
    $file = fopen( $GLOBALS["htgroupfile"], "r" );
    while( $file and ! feof( $file ) ) {
      $row = fgets( $file );
      if( ereg( "^([A-Za-z0-9_]*): *(($GLOBALS[username_ereg][, ]*)*)[ \n]*$", $row, $regs ) ) {
        if( ereg( "^ *$", $regs[2] ) )
          $groups[$regs[1]] = array();
        else
          $groups[$regs[1]] = split( "[, ] *", $regs[2] );
      }
    }
    fclose( $file );
    $GLOBALS["groups"] = $groups;
  }
  return $groups;
}

function write_groups( $groups ) {
  $file = fopen( $GLOBALS["htgroupfile"], "w" );
  foreach( $groups as $groupname => $users ) {
    $row = "$groupname:" . implode( " ", $users ) . "\n";
    fputs( $file, $row );
  }
  fclose( $file );
  $GLOBALS["groups"] = $groups;
}

function get_dirs() {
  $dirs = $GLOBALS["dirs"];
  if( count( $dirs ) < 1 ) {
    $dirs = array();
    $basedir = $GLOBALS["basedir"];
    $baselen = strlen($basedir);
    $digdirs = array( $basedir );
    while( count( $digdirs ) > 0 ) {
      $digdir = array_shift( $digdirs );
      $d = @opendir( $digdir );
      if( ! $d ) continue;
      while( ( $file = readdir($d) ) !== false ) {
	if( substr( $file, 0, 1 ) == "." )
	  continue;
	$fullname = "$digdir/$file";
	if( ! is_dir( $fullname ) )
	  continue;
	array_push( $digdirs, $fullname );
	array_push( $dirs, substr( $fullname, $baselen ) );
      }
      closedir( $d );
    }
    if( ! sort($dirs) )
	err("sort failed");
    $GLOBALS["dirs"] = $dirs;
  }
  return $dirs;
}

function get_dir_group( $dir ) {
  $file = "$GLOBALS[basedir]$dir/.htaccess";
  $group = "";
  if( file_exists($file) ) {
    $f = fopen($file,"r");
    while( $f and ! feof($f) ) {
      $row = fgets( $f );
      if( ereg( "^ *require group  *([A-Za-z0-9_]*)", $row, $regs ) ) {
	$group = $regs[1];
        break;
      }
    }
    @fclose( $f );
  }
  return $group;
}

function set_dir_group( $dir, $group ) {
  $basedir = $GLOBALS["basedir"];
  if( substr( $dir, 0, strlen($basedir) ) != $basedir )
    $fulldir = "$basedir$dir";
  else
    $fulldir = $dir;
  if( ! is_dir($fulldir) )
    return err("No such directory: $dir");
  $file = "$fulldir/.htaccess";
  if( ! file_exists($file) ) {
    if( ! $group )
      return;
    $fh = @fopen( $file, "w");
    @fputs( $fh, $GLOBALS["htaccess_content"] );
    @fclose($file);
    err("Access file created for directory $dir");
  }
  $from = @fopen( $file, "r" );
  $to = @fopen( "$file+", "w" );
  if( ! $from or ! $to )
    return err("Failed to modify directory access file");
  $c = $group ? "" : "#";
  $replace = "\\1$c\\2 $group";
  while( $from and ! feof( $from ) ) {
	$row = ereg_replace( "^( *)#?( *require group) *[A-Za-z0-9_]*", 
			     $replace, fgets( $from ) );
	fputs( $to, $row );
  }
  @fclose( $from );
  @fclose( $to );
  if( file_exists( "$file-" ) )
    unlink( "$file-" );
  rename( $file, "$file-" );
  rename( "$file+", $file );
  return true;
}

function get_phrase( $tag ) {
  $value = $GLOBALS["phrase"][$tag];
  if( $value )
    return $value;
  else
    return $tag;
}

function phrase( $tag ) {
  echo get_phrase( $tag );
}

function err( $msg ) {
  $GLOBALS["status"] .= get_phrase( $msg ) . "<br/>\n";
  return false;
}

function print_status() {
  if( $GLOBALS["status"] )
    echo "\n<font color=red><b>" . get_phrase('status') . 
      ": $GLOBALS[status]</b></font>\n";
}

function print_menu() {
  echo "\n<ul>";
  foreach( $GLOBALS["menu"] as $phrase => $url ) {
    echo "<li> <a href=\"$url\">" . get_phrase($phrase) . "</a></li>\n";
  }
  echo "</ul>\n";
}

function print_top() {
  print_menu();
  print_status();
}

$control_user = getenv("REMOTE_USER");

if( ! ok_username( $control_user ) ) {
    echo "<b>User authentication failed.\n</b>";
    error_log( "$HTTP_HOST $REQUEST_URI: User authentication failed.\n" );
}

?>
